==> json-joy-2.8.0.tgz: package/esm/json-crdt/codec/json/Decoder.js <==
import { LogicalTimestamp, VectorClock } from '../../../json-crdt-patch/clock';
import { ORIGIN } from '../../../json-crdt-patch/constants';
import { FALSE, NULL, TRUE, UNDEFINED } from '../../constants';
import { Document } from '../../document';
import { ConstantType } from '../../types/const/ConstantType';
import { DocRootType } from '../../types/lww-doc-root/DocRootType';
import { ObjectChunk } from '../../types/lww-object/ObjectChunk';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ValueType } from '../../types/lww-value/ValueType';
import { ArrayChunk } from '../../types/rga-array/ArrayChunk';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { StringChunk } from '../../types/rga-string/StringChunk';
import { StringType } from '../../types/rga-string/StringType';
export class Decoder {
    decode({ clock, root }) {
        const vectorClock = this.decodeClock(clock);
        const doc = new Document(vectorClock);
        this.decodeRoot(doc, root);
        return doc;
    }
    decodeClock(timestamps) {
        const [ts] = timestamps;
        const vectorClock = new VectorClock(ts[0], ts[1]);
        const length = timestamps.length;
        for (let i = 1; i < length; i++) {
            const ts = timestamps[i];
            const [sessionId, time] = ts;
            vectorClock.clocks.set(sessionId, new LogicalTimestamp(sessionId, time));
        }
        return vectorClock;
    }
    decodeTimestamp([sessionId, time]) {
        return new LogicalTimestamp(sessionId, time);
    }
    decodeRoot(doc, { id, node }) {
        const ts = this.decodeTimestamp(id);
        const jsonNode = node ? this.decodeNode(doc, node) : null;
        const root = new DocRootType(doc, ts, jsonNode);
        doc.root = root;
    }
    decodeNode(doc, node) {
        switch (node.type) {
            case 'obj': return this.decodeObj(doc, node);
            case 'arr': return this.decodeArr(doc, node);
            case 'str': return this.decodeStr(doc, node);
            case 'val': return this.decodeVal(doc, node);
            case 'const': return this.decodeConst(doc, node);
        }
        throw new Error('UNKNOWN_NODE');
    }
    decodeObj(doc, node) {
        const id = this.decodeTimestamp(node.id);
        const obj = new ObjectType(doc, id);
        const keys = Object.keys(node.chunks);
        for (const key of keys) {
            const val = node.chunks[key];
            obj.putChunk(key, new ObjectChunk(this.decodeTimestamp(val.id), this.decodeNode(doc, val.node)));
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeArr(doc, node) {
        const obj = new ArrayType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeArrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeArrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new ArrayChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new ArrayChunk(id, c.nodes.map(n => this.decodeNode(doc, n)));
    }
    decodeStr(doc, node) {
        const obj = new StringType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeStrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeStrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new StringChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new StringChunk(id, c.value);
    }
    decodeVal(doc, node) {
        const obj = new ValueType(this.decodeTimestamp(node.id), this.decodeTimestamp(node.writeId), node.value);
        doc.nodes.index(obj);
        return obj;
    }
    decodeConst(doc, node) {
        switch (node.value) {
            case null: return NULL;
            case true: return TRUE;
            case false: return FALSE;
            case undefined: return UNDEFINED;
        }
        return new ConstantType(ORIGIN, node.value);
    }
}

==> json-joy-7.6.0.tgz: package/esm/json-crdt/codec/json/Decoder.js <==
import { ArrayChunk } from '../../types/rga-array/ArrayChunk';
import { ArrayType } from '../../types/rga-array/ArrayType';
import { BinaryChunk } from '../../types/rga-binary/BinaryChunk';
import { BinaryType } from '../../types/rga-binary/BinaryType';
import { ConstantType } from '../../types/const/ConstantType';
import { DocRootType } from '../../types/lww-doc-root/DocRootType';
import { FALSE, NULL, TRUE, UNDEFINED } from '../../constants';
import { fromBase64 } from '../../../util/base64/decode';
import { LogicalTimestamp, LogicalVectorClock, ServerVectorClock } from '../../../json-crdt-patch/clock';
import { Model } from '../../model';
import { ObjectChunk } from '../../types/lww-object/ObjectChunk';
import { ObjectType } from '../../types/lww-object/ObjectType';
import { ORIGIN } from '../../../json-crdt-patch/constants';
import { StringChunk } from '../../types/rga-string/StringChunk';
import { StringType } from '../../types/rga-string/StringType';
import { ValueType } from '../../types/lww-value/ValueType';
export class Decoder {
    decode({ time, root }) {
        const isServerClock = typeof time === 'number';
        const doc = isServerClock
            ? Model.withServerClock(time)
            : Model.withLogicalClock(this.decodeClock(time));
        this.decodeRoot(doc, root);
        return doc;
    }
    decodeClock(timestamps) {
        const [ts] = timestamps;
        const vectorClock = new LogicalVectorClock(ts[0], ts[1]);
        const length = timestamps.length;
        for (let i = 0; i < length; i++) {
            const ts = timestamps[i];
            const [sessionId, time] = ts;
            vectorClock.observe(new LogicalTimestamp(sessionId, time), 1);
        }
        return vectorClock;
    }
    decodeTimestamp(ts) {
        const isServerClock = typeof ts === 'number';
        return isServerClock
            ? new ServerVectorClock(ts)
            : new LogicalTimestamp(ts[0], ts[1]);
    }
    decodeRoot(doc, { id, node }) {
        const ts = this.decodeTimestamp(id);
        const jsonNode = node ? this.decodeNode(doc, node) : null;
        const root = new DocRootType(doc, ts, jsonNode);
        doc.root = root;
    }
    decodeNode(doc, node) {
        switch (node.type) {
            case 'obj':
                return this.decodeObj(doc, node);
            case 'arr':
                return this.decodeArr(doc, node);
            case 'str':
                return this.decodeStr(doc, node);
            case 'val':
                return this.decodeVal(doc, node);
            case 'const':
                return this.decodeConst(doc, node);
            case 'bin':
                return this.decodeBin(doc, node);
        }
        throw new Error('UNKNOWN_NODE');
    }
    decodeObj(doc, node) {
        const id = this.decodeTimestamp(node.id);
        const obj = new ObjectType(doc, id);
        const keys = Object.keys(node.chunks);
        for (const key of keys) {
            const val = node.chunks[key];
            obj.putChunk(key, new ObjectChunk(this.decodeTimestamp(val.id), this.decodeNode(doc, val.node)));
        }
        doc.nodes.index(obj);
        return obj;
    }
    decodeArr(doc, node) {
        const obj = new ArrayType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeArrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeArrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new ArrayChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new ArrayChunk(id, c.nodes.map((n) => this.decodeNode(doc, n)));
    }
    decodeStr(doc, node) {
        const obj = new StringType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeStrChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeStrChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new StringChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else
            return new StringChunk(id, c.value);
    }
    decodeBin(doc, node) {
        const obj = new BinaryType(doc, this.decodeTimestamp(node.id));
        for (const c of node.chunks)
            obj.append(this.decodeBinChunk(doc, c));
        doc.nodes.index(obj);
        return obj;
    }
    decodeBinChunk(doc, c) {
        const id = this.decodeTimestamp(c.id);
        if (typeof c.span === 'number') {
            const chunk = new BinaryChunk(id, undefined);
            chunk.deleted = c.span;
            return chunk;
        }
        else {
            const data = fromBase64(c.value);
            return new BinaryChunk(id, data);
        }
    }
    decodeVal(doc, node) {
        const obj = new ValueType(this.decodeTimestamp(node.id), this.decodeTimestamp(node.writeId), node.value);
        doc.nodes.index(obj);
        return obj;
    }
    decodeConst(doc, node) {
        switch (node.value) {
            case null:
                return NULL;
            case true:
                return TRUE;
            case false:
                return FALSE;
            case undefined:
                return UNDEFINED;
        }
        return new ConstantType(ORIGIN, node.value);
    }
}
